#!/usr/bin/env python
#
# Copyright (C) 2011-2013 Calum Lind <calumlind@gmail.com>
# Copyright (C) 2009 Andrew Resch <andrewresch@gmail.com>
#
# This file is part of Deluge and is licensed under GNU General Public License 3.0, or later, with
# the additional special exception to link portions of this program with the OpenSSL library.
# See LICENSE for more details.
#

"""Parses Python and Javascript code for translation strings to create the 'deluge.pot' template for translators"""

import os
import re
from datetime import datetime
from subprocess import call

from version import get_version

# Paths to exclude
EXCLUSIONS = ['deluge/scripts', 'deluge/i18n', 'deluge/tests']
WEBUI_JS_DIR = 'deluge/ui/web/js/deluge-all'
WEBUI_RENDER_DIR = 'deluge/ui/web/render'
INFILES_LIST = 'infiles.list'
POT_FILEPATH = os.path.join('deluge', 'i18n', 'deluge.pot')

RE_EXC_PLUGIN_BUILD = re.compile('deluge\\/plugins\\/.*\\/build')

xgettext_cmd = [
    'xgettext',
    '--from-code=UTF-8',
    '-kN_:1',
    '-f%s' % INFILES_LIST,
    '-o%s' % POT_FILEPATH,
    '--package-name=%s' % 'Deluge',
    '--copyright-holder=%s' % 'Deluge Team',
    '--package-version=%s' % get_version(prefix='deluge-', suffix='.dev0'),
    '--msgid-bugs-address=%s' % 'http://deluge-torrent.org',
]

to_translate = []
for dirpath, dirnames, filenames in os.walk('deluge'):
    for filename in filenames:
        if dirpath not in EXCLUSIONS and not RE_EXC_PLUGIN_BUILD.match(dirpath):
            filepath = os.path.join(dirpath, filename)
            if os.path.splitext(filepath)[1] in ('.py', '.glade'):
                to_translate.append(filepath)
            else:
                if filename.endswith('.xml.in'):
                    gtxt_type = 'gettext/xml'
                elif filename.endswith('.in'):
                    gtxt_type = 'gettext/ini'
                elif filename.endswith('.ui'):
                    gtxt_type = 'gettext/glade'
                else:
                    continue
                call(['intltool-extract', '--quiet', '--type=%s' % gtxt_type, filepath])
                filepath += '.h'
            to_translate.append(filepath)

with open(INFILES_LIST, 'w') as f:
    for line in to_translate:
        f.write(line + '\n')

# Create pot file from file list
call(xgettext_cmd)

# find javascript files
js_to_translate = []
for dirpath, dirnames, filenames in os.walk(WEBUI_JS_DIR):
    for filename in filenames:
        if os.path.splitext(filename)[1] == '.js':
            js_to_translate.append(os.path.join(dirpath, filename))

# find render html files
for dirpath, dirnames, filenames in os.walk(WEBUI_RENDER_DIR):
    for filename in filenames:
        if os.path.splitext(filename)[1] == '.html':
            js_to_translate.append(os.path.join(dirpath, filename))

with open(INFILES_LIST, 'w') as f:
    for line in js_to_translate:
        f.write(line + '\n')

# Force xgettext language to parse javascript and update pot file
# Note: For javascript files xgettext will parse comments, so single apostrophes or quotes are
# flagged as a 'warning: untermined string'. Either ignore warning or edit javascript comment.
call(xgettext_cmd + ['--language=Python', '-j'])

# Replace YEAR and PACKAGE in the copyright message
with open(POT_FILEPATH) as f:
    lines = f.readlines()
with open(POT_FILEPATH, 'w') as f:
    for line in lines:
        if 'YEAR' in line:
            line = line.replace('YEAR', str(datetime.now().year))
        elif 'PACKAGE' in line:
            line = line.replace('PACKAGE', 'Deluge')
        f.write(line)

# Clean up temp files
os.remove(INFILES_LIST)
for filepath in to_translate:
    if filepath.endswith('.h'):
        os.remove(filepath)

print('Created %s' % POT_FILEPATH)



#
# This file is part of Deluge and is licensed under GNU General Public License 3.0, or later, with
# the additional special exception to link portions of this program with the OpenSSL library.
# See LICENSE for more details.
#

import os
import tempfile

import pytest

from deluge import metafile
from deluge._libtorrent import LT_VERSION
from deluge.common import VersionSplit

from . import common


def check_torrent(filename):
    # Test loading with libtorrent to make sure it's valid
    from deluge._libtorrent import lt

    lt.torrent_info(filename)

    # Test loading with our internal TorrentInfo class
    from deluge.ui.common import TorrentInfo

    TorrentInfo(filename)


class TestMetafile:
    def test_save_multifile(self):
        # Create a temporary folder for torrent creation
        tmp_path = tempfile.mkdtemp()
        with open(os.path.join(tmp_path, 'file_A'), 'wb') as tmp_file:
            tmp_file.write(b'a' * (312 * 1024))
        with open(os.path.join(tmp_path, 'file_B'), 'wb') as tmp_file:
            tmp_file.write(b'b' * (2354 * 1024))
        with open(os.path.join(tmp_path, 'file_C'), 'wb') as tmp_file:
            tmp_file.write(b'c' * (11 * 1024))

        tmp_fd, tmp_file = tempfile.mkstemp('.torrent')
        metafile.make_meta_file(tmp_path, '', 32768, target=tmp_file)

        check_torrent(tmp_file)

        os.remove(os.path.join(tmp_path, 'file_A'))
        os.remove(os.path.join(tmp_path, 'file_B'))
        os.remove(os.path.join(tmp_path, 'file_C'))
        os.rmdir(tmp_path)
        os.close(tmp_fd)
        os.remove(tmp_file)

    def test_save_singlefile(self):
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_data = tmp_dir + '/testdata'
            with open(tmp_data, 'wb') as tmp_file:
                tmp_file.write(b'a' * (2314 * 1024))

            tmp_torrent = tmp_dir + '/.torrent'
            metafile.make_meta_file(tmp_data, '', 32768, target=tmp_torrent)

            check_torrent(tmp_torrent)

    @pytest.mark.parametrize(
        'path',
        [
            common.get_test_data_file('deluge.png'),
            common.get_test_data_file('unicode_filenames.torrent'),
            os.path.dirname(common.get_test_data_file('deluge.png')),
        ],
    )
    @pytest.mark.parametrize(
        'torrent_format',
        [
            metafile.TorrentFormat.V1,
            metafile.TorrentFormat.V2,
            metafile.TorrentFormat.HYBRID,
        ],
    )
    @pytest.mark.parametrize('piece_length', [2**14, 2**15, 2**16])
    @pytest.mark.parametrize('private', [True, False])
    def test_create_info(self, path, torrent_format, piece_length, private):
        our_info, our_piece_layers = metafile.makeinfo(
            path,
            piece_length,
            metafile.dummy,
            private=private,
            torrent_format=torrent_format,
        )
        lt_info, lt_piece_layers = metafile.makeinfo_lt(
            path,
            piece_length,
            private=private,
            torrent_format=torrent_format,
        )

        if (
            torrent_format == metafile.TorrentFormat.HYBRID
            and os.path.isdir(path)
            and VersionSplit(LT_VERSION) <= VersionSplit('2.0.7.0')
        ):
            # Libtorrent didn't correctly follow the standard until version 2.0.7 included
            # https://github.com/arvidn/libtorrent/commit/74d82a0cd7c2e9e3c4294901d7eb65e247050df4
            # If last file is a padding, ignore that file and the last piece.
            if our_info[b'files'][-1][b'path'][0] == b'.pad':
                our_info[b'files'] = our_info[b'files'][:-1]
                our_info[b'pieces'] = our_info[b'pieces'][:-32]
                lt_info[b'pieces'] = lt_info[b'pieces'][:-32]

        assert our_info == lt_info
        assert our_piece_layers == lt_piece_layers



#
# This file is part of Deluge and is licensed under GNU General Public License 3.0, or later, with
# the additional special exception to link portions of this program with the OpenSSL library.
# See LICENSE for more details.
#
import time
from unittest.mock import Mock

import pytest
import pytest_twisted
from twisted.internet import defer, threads

import deluge.component as component


class ComponentTester(component.Component):
    def __init__(self, name, depend=None):
        super().__init__(name, depend=depend)
        event_methods = ('start', 'update', 'pause', 'resume', 'stop', 'shutdown')
        for event_method in event_methods:
            setattr(self, event_method, Mock())


class ComponentTesterDelayStart(ComponentTester):
    def __init__(self, name, depend=None):
        super().__init__(name, depend=depend)
        self.start = Mock(side_effect=self.delay)

    @pytest_twisted.inlineCallbacks
    def delay(self):
        yield threads.deferToThread(time.sleep, 0.5)


@pytest.mark.usefixtures('component')
class TestComponent:
    async def test_start_component(self):
        c = ComponentTester('test_start')
        await component.start(['test_start'])

        assert c._component_state == 'Started'
        assert c.start.call_count == 1

    async def test_start_stop_depends(self):
        c1 = ComponentTester('test_start_depends_c1')
        c2 = ComponentTester('test_start_depends_c2', depend=['test_start_depends_c1'])

        await component.start('test_start_depends_c2')

        assert c1._component_state == 'Started'
        assert c2._component_state == 'Started'
        assert c1.start.call_count == 1
        assert c2.start.call_count == 1

        await component.stop(['test_start_depends_c1'])

        assert c1._component_state == 'Stopped'
        assert c2._component_state == 'Stopped'
        assert c1.stop.call_count == 1
        assert c2.stop.call_count == 1

    async def start_with_depends(self):
        c1 = ComponentTesterDelayStart('test_start_all_c1')
        c2 = ComponentTester('test_start_all_c2', depend=['test_start_all_c4'])
        c3 = ComponentTesterDelayStart(
            'test_start_all_c3', depend=['test_start_all_c5', 'test_start_all_c1']
        )
        c4 = ComponentTester('test_start_all_c4', depend=['test_start_all_c3'])
        c5 = ComponentTester('test_start_all_c5')

        await component.start()
        return c1, c2, c3, c4, c5

    def finish_start_with_depends(self, *args):
        for c in args[1:]:
            component.deregister(c)

    async def test_start_all(self):
        components = await self.start_with_depends()
        for c in components:
            assert c._component_state == 'Started'
            assert c.start.call_count == 1

        self.finish_start_with_depends(components)

    def test_register_exception(self):
        ComponentTester('test_register_exception')
        with pytest.raises(component.ComponentAlreadyRegistered):
            ComponentTester(
                'test_register_exception',
            )

    async def test_stop(self):
        c = ComponentTester('test_stop')

        await component.start(['test_stop'])

        assert c._component_state == 'Started'

        await component.stop(['test_stop'])

        assert c._component_state == 'Stopped'
        assert not c._component_timer.running
        assert c.stop.call_count == 1

    async def test_stop_all(self):
        components = await self.start_with_depends()
        assert all(c._component_state == 'Started' for c in components)

        component.stop()
        for c in components:
            assert c._component_state == 'Stopped'
            assert c.stop.call_count == 1

        self.finish_start_with_depends(components)

    async def test_update(self):
        c = ComponentTester('test_update')
        init_update_count = int(c.update.call_count)
        await component.start(['test_update'])

        assert c._component_timer
        assert c._component_timer.running
        assert c.update.call_count != init_update_count
        await component.stop()

    async def test_pause(self):
        c = ComponentTester('test_pause')
        init_update_count = int(c.update.call_count)

        await component.start(['test_pause'])

        assert c._component_timer
        assert c.update.call_count != init_update_count

        await component.pause(['test_pause'])

        assert c._component_state == 'Paused'
        assert c.pause.call_count == 1
        assert c.update.call_count != init_update_count
        assert not c._component_timer.running

    async def test_resume(self):
        c = ComponentTester('test_resume')

        await component.start(['test_resume'])

        assert c._component_state == 'Started'

        await component.pause(['test_resume'])

        assert c._component_state == 'Paused'

        await component.resume(['test_resume'])

        assert c._component_state == 'Started'
        assert c.resume.call_count == 1
        assert c._component_timer.running

    async def test_component_start_error(self):
        ComponentTester('test_start_error')
        await component.start(['test_start_error'])
        await component.pause(['test_start_error'])
        test_comp = component.get('test_start_error')
        with pytest.raises(component.ComponentException, match='Current state: Paused'):
            await test_comp._component_start()

    async def test_start_paused_error(self):
        name = 'test_pause_error'
        ComponentTester(name)
        await component.start([name])
        await component.pause([name])

        (failure, error), *_ = await component.start()
        assert (failure, error.type, error.value.message) == (
            defer.FAILURE,
            component.ComponentException,
            (
                f'Trying to start component "{name}" but it is '
                'not in a stopped state. Current state: Paused'
            ),
        )

    async def test_shutdown(self):
        c = ComponentTester('test_shutdown')

        await component.start(['test_shutdown'])
        await component.shutdown()

        assert c.shutdown.call_count == 1
        assert c._component_state == 'Stopped'
        assert not c._component_timer.running
        assert c.stop.call_count == 1



#!/usr/bin/python3
#
# Create Deluge PNG icons from SVG
#
# Required image tools:
#  * rsvg-convert
#  * convert (ImageMagik)
#  * oxipng
#  * pngquant
#
import shutil
import subprocess
from dataclasses import dataclass, field
from pathlib import Path


@dataclass
class IconPack:
    name: str
    dir: Path
    icon_sizes: list[int]
    panel_sizes: list[int]
    ico_sizes: list[int]
    pixmaps_dir: Path = field(init=False)
    theme_dir: Path = field(init=False)
    theme_svg: Path = field(init=False)
    theme_pngs: dict[int, Path] = field(init=False)
    logo_svg: Path = field(init=False)
    logo_ico: Path = field(init=False)
    logo_png: Path = field(init=False)

    def __post_init__(self):
        self.pixmaps_dir = self.dir / 'pixmaps'
        self.logo_svg = self.pixmaps_dir / f'{self.name}.svg'
        self.logo_ico = self.pixmaps_dir / f'{self.name}.ico'
        self.logo_png = self.pixmaps_dir / f'{self.name}.png'

        self.theme_dir = self.dir / 'icons' / 'hicolor'
        self.theme_svg = self.theme_dir / 'scalable' / 'apps' / f'{self.name}.svg'
        self.theme_pngs = self.create_theme_pngs_paths(
            self.name, self.icon_sizes, self.theme_dir
        )

    @staticmethod
    def create_theme_pngs_paths(name, icon_sizes, out_dir):
        return {
            size: out_dir / f'{size}x{size}' / 'apps' / f'{name}.png'
            for size in icon_sizes
        }


@dataclass
class WebIconPack:
    name: str
    dir: Path
    icon_sizes: list[int]
    favicon_sizes: list[int]
    icons_dir: Path = field(init=False)
    touch: Path = field(init=False)
    favicon: Path = field(init=False)

    def __post_init__(self):
        self.icons_dir = self.dir / 'icons'
        self.touch = self.icons_dir / f'{self.name}-apple-180.png'
        self.favicon = self.icons_dir / 'favicon.ico'


def convert_svg_to_png(svg_file, png_file, size, background_color=None):
    rsvg_options = [
        '-w',
        str(size),
        '-h',
        str(size),
        '-o',
        png_file,
    ]
    rsvg_options + ['-b', {background_color}] if background_color else []

    subprocess.run(['rsvg-convert'] + rsvg_options + [svg_file], check=True)


def compress_png(png_file):
    subprocess.run(
        ['pngquant', '--quality=70-95', '--ext', '.png', '--force', png_file],
        check=True,
    )
    subprocess.run(['oxipng', png_file], check=True)


def create_panel_icons(icon_pack, sizes):
    for size in sizes:
        app_png = icon_pack[size]
        panel_png = app_png.with_name(f'{app_png.stem}-panel.png')
        shutil.copyfile(app_png, panel_png)


def create_hicolor_icons(svg_icon, icon_pack):
    """Convert SVG icon to hicolor PNG icons."""
    for size, png_file in icon_pack.items():
        png_file.parent.mkdir(parents=True, exist_ok=True)
        convert_svg_to_png(svg_icon, png_file, size)
        compress_png(png_file)


def create_ico_icon(icon_pack, sizes, ico_file):
    infiles = [icon_pack[size] for size in sizes]
    ico_file.parent.mkdir(parents=True, exist_ok=True)

    subprocess.run(['convert', *infiles, ico_file], check=True)


def create_hicolor_svg(src_svg, dest_svg):
    dest_svg.parent.mkdir(parents=True, exist_ok=True)
    shutil.copyfile(src_svg, dest_svg)


def create_mini_icons(pixmaps_dir):
    pixmap_svgs = pixmaps_dir.glob('*.svg')

    for svg_file in pixmap_svgs:
        png_file = pixmaps_dir / f'{svg_file.stem}16.png'
        convert_svg_to_png(svg_file, png_file, 16)
        compress_png(png_file)


def create_logo(deluge_png, pixmap_png):
    pixmap_png.parent.mkdir(parents=True, exist_ok=True)
    shutil.copyfile(deluge_png, pixmap_png)


def create_web_status_icons(src_dir: Path, dest_dir: Path):
    """Web UI status icons from 16px icons."""
    pngs_16px = src_dir.glob('*16.png')
    dest_dir.mkdir(parents=True, exist_ok=True)
    for path in pngs_16px:
        if path.stem.startswith('tracker'):
            continue
        new_name = path.stem.replace('16', '') + '.png'
        shutil.copyfile(path, dest_dir / new_name)


def create_touch_icon(svg_file, png_file, size):
    """Web icons with background color for Apple or Android"""
    png_file.parent.mkdir(parents=True, exist_ok=True)
    convert_svg_to_png(svg_file, png_file, size, background_color='#599EEE')
    compress_png(png_file)


def create_web_icons(app_pngs, sizes, dest_dir):
    dest_dir.mkdir(parents=True, exist_ok=True)
    for size in sizes:
        app_png = app_pngs[size]
        web_png = dest_dir / f'{app_png.stem}-{size}.png'
        shutil.copyfile(app_png, web_png)


def main():
    data_dir = Path.cwd() / 'deluge' / 'ui' / 'data'
    if not data_dir.is_dir():
        exit(f'No path to UI data dir: {data_dir}')

    # Create Deluge UI icons
    icon_pack_sizes = [16, 22, 24, 32, 36, 48, 64, 72, 96, 128, 192, 256, 512]
    panel_icon_sizes = [16, 22, 24]
    ico_icon_sizes = [16, 32, 48, 64, 128, 256]
    ui_icons = IconPack(
        name='deluge',
        dir=data_dir,
        icon_sizes=icon_pack_sizes,
        panel_sizes=panel_icon_sizes,
        ico_sizes=ico_icon_sizes,
    )

    # Theme icons for GTK
    create_hicolor_icons(ui_icons.logo_svg, ui_icons.theme_pngs)
    create_hicolor_svg(ui_icons.logo_svg, ui_icons.theme_svg)
    create_mini_icons(ui_icons.pixmaps_dir)
    # Panel icon for systray
    create_panel_icons(ui_icons.theme_pngs, ui_icons.panel_sizes)

    # Deluge logos
    create_ico_icon(ui_icons.theme_pngs, ui_icons.ico_sizes, ui_icons.logo_ico)
    create_logo(ui_icons.theme_pngs[48], ui_icons.logo_png)

    # Web UI Icons
    web_icon_sizes = [32, 192, 512]
    favicon_sizes = [16, 32, 48]
    web_icons = WebIconPack(
        name='deluge',
        dir=data_dir / '..' / 'web',
        icon_sizes=web_icon_sizes,
        favicon_sizes=favicon_sizes,
    )
    create_web_icons(ui_icons.theme_pngs, web_icons.icon_sizes, web_icons.icons_dir)
    create_web_status_icons(ui_icons.pixmaps_dir, web_icons.icons_dir)
    create_touch_icon(ui_icons.logo_svg, web_icons.touch, 180)
    create_ico_icon(ui_icons.theme_pngs, web_icons.favicon_sizes, web_icons.favicon)


if __name__ == '__main__':
    main()



#
# This file is part of Deluge and is licensed under GNU General Public License 3.0, or later, with
# the additional special exception to link portions of this program with the OpenSSL library.
# See LICENSE for more details.
#
import itertools
import os
import time
from base64 import b64encode
from unittest import mock

import pytest
from twisted.internet import defer, reactor
from twisted.internet.task import deferLater

import deluge.component as component
import deluge.core.torrent
import deluge.tests.common as common
from deluge._libtorrent import lt
from deluge.common import VersionSplit, utf8_encode_structure
from deluge.conftest import BaseTestCase
from deluge.core.core import Core
from deluge.core.rpcserver import RPCServer
from deluge.core.torrent import Torrent
from deluge.core.torrentmanager import TorrentManager, TorrentState

try:
    from unittest.mock import AsyncMock
except ImportError:
    from mock import AsyncMock


class TestTorrent(BaseTestCase):
    def setup_config(self):
        core_config = deluge.config.Config(
            'core.conf',
            defaults=deluge.core.preferencesmanager.DEFAULT_PREFS,
            config_dir=self.config_dir,
        )
        core_config.save()

    def set_up(self):
        self.setup_config()
        self.rpcserver = RPCServer(listen=False)
        self.core = Core()
        self.core.config.config['lsd'] = False
        self.core.config.config['new_release_check'] = False
        self.session = self.core.session
        self.torrent = None
        return component.start()

    def tear_down(self):
        def on_shutdown(result):
            del self.rpcserver
            del self.core

        return component.shutdown().addCallback(on_shutdown)

    def print_priority_list(self, priorities):
        tmp = ''
        for i, p in enumerate(priorities):
            if i % 100 == 0:
                print(tmp)
                tmp = ''
            tmp += '%s' % p
        print(tmp)

    def assert_state(self, torrent, state):
        """Assert torrent state matches expected state"""
        torrent.update_state()
        assert torrent.state == state

    def assert_state_wait(self, torrent, expected, timeout=1, interval=0.2):
        """Assert state but retry with timeout e.g. Allow for async lt alerts"""
        start = time.time()

        while time.time() - start < timeout:
            torrent.update_state()
            time.sleep(interval)
            if torrent.state == expected:
                break
        else:
            assert torrent.state == expected

    def get_torrent_atp(self, filename):
        filename = common.get_test_data_file(filename)
        with open(filename, 'rb') as _file:
            info = lt.torrent_info(lt.bdecode(_file.read()))
        atp = {
            'ti': info,
            'save_path': os.getcwd(),
            'storage_mode': lt.storage_mode_t.storage_mode_sparse,
            'flags': (
                lt.torrent_flags.auto_managed
                | lt.torrent_flags.duplicate_is_error & ~lt.torrent_flags.paused
            ),
        }
        return atp

    async def test_set_file_priorities(self):
        if getattr(lt, 'file_prio_alert', None):
            # Libtorrent 2.0.3 and later has a file_prio_alert
            prios_set = defer.Deferred()
            prios_set.addTimeout(1.5, reactor)
            component.get('AlertManager').register_handler(
                'file_prio_alert', lambda a: prios_set.callback(True)
            )
        else:
            # On older libtorrent, we just wait a while
            prios_set = deferLater(reactor, 0.8)

        atp = self.get_torrent_atp('dir_with_6_files.torrent')
        handle = self.session.add_torrent(atp)
        torrent = Torrent(handle, {})

        result = torrent.get_file_priorities()
        assert all(x == 4 for x in result)

        new_priorities = [3, 1, 2, 0, 5, 6, 7]
        torrent.set_file_priorities(new_priorities)
        assert torrent.get_file_priorities() == new_priorities

        # Test with handle.piece_priorities as handle.file_priorities async
        # updates and will return old value. Also need to remove a priority
        # value as one file is much smaller than piece size so doesn't show.
        await prios_set  # Delay to wait for alert from lt
        piece_prio = handle.get_piece_priorities()
        result = all(p in piece_prio for p in [3, 2, 0, 5, 6, 7])
        assert result

    def test_set_prioritize_first_last_pieces(self):
        piece_indexes = [
            0,
            1,
            50,
            51,
            52,
            110,
            111,
            112,
            113,
            200,
            201,
            202,
            212,
            213,
            214,
            215,
            216,
            217,
            457,
            458,
            459,
            460,
            461,
            462,
        ]
        self.run_test_set_prioritize_first_last_pieces(
            'dir_with_6_files.torrent', piece_indexes
        )

    def run_test_set_prioritize_first_last_pieces(
        self, torrent_file, prioritized_piece_indexes
    ):
        atp = self.get_torrent_atp(torrent_file)
        handle = self.session.add_torrent(atp)

        self.torrent = Torrent(handle, {})
        priorities_original = handle.get_piece_priorities()
        self.torrent.set_prioritize_first_last_pieces(True)
        priorities = handle.get_piece_priorities()

        # The length of the list of new priorites is the same as the original
        assert len(priorities_original) == len(priorities)

        # Test the priority of all the pieces against the calculated indexes.
        for idx, priority in enumerate(priorities):
            if idx in prioritized_piece_indexes:
                assert priorities[idx] == 7
            else:
                assert priorities[idx] == 4

        # self.print_priority_list(priorities)

    def test_set_prioritize_first_last_pieces_false(self):
        atp = self.get_torrent_atp('dir_with_6_files.torrent')
        handle = self.session.add_torrent(atp)
        self.torrent = Torrent(handle, {})
        # First set some pieces prioritized
        self.torrent.set_prioritize_first_last_pieces(True)
        # Reset pirorities
        self.torrent.set_prioritize_first_last_pieces(False)
        priorities = handle.get_piece_priorities()

        # Test the priority of the prioritized pieces
        for i in priorities:
            assert priorities[i] == 4

        # self.print_priority_list(priorities)

    def test_torrent_error_data_missing(self):
        options = {'seed_mode': True}
        filename = common.get_test_data_file('test_torrent.file.torrent')
        with open(filename, 'rb') as _file:
            filedump = b64encode(_file.read())
        torrent_id = self.core.add_torrent_file(filename, filedump, options)
        torrent = self.core.torrentmanager.torrents[torrent_id]

        # Inital check will fail and return to download state
        self.assert_state_wait(torrent, 'Downloading')

        # Force an error by reading (non-existant) piece from disk
        torrent.handle.read_piece(0)
        self.assert_state_wait(torrent, 'Error')

    def test_torrent_error_resume_original_state(self):
        options = {'seed_mode': True, 'add_paused': True}
        filename = common.get_test_data_file('test_torrent.file.torrent')
        with open(filename, 'rb') as _file:
            filedump = b64encode(_file.read())
        torrent_id = self.core.add_torrent_file(filename, filedump, options)
        torrent = self.core.torrentmanager.torrents[torrent_id]

        orig_state = 'Paused'
        self.assert_state(torrent, orig_state)

        # Force an error by reading (non-existant) piece from disk
        torrent.handle.read_piece(0)
        self.assert_state_wait(torrent, 'Error')

        # Clear error and verify returned to original state
        torrent.force_recheck()

    def test_torrent_error_resume_data_unaltered(self):
        if VersionSplit(lt.__version__) >= VersionSplit('1.2.0.0'):
            pytest.skip('Test not working as expected on lt 1.2 or greater')

        resume_data = {
            'active_time': 13399,
            'num_incomplete': 16777215,
            'announce_to_lsd': 1,
            'seed_mode': 0,
            'pieces': '\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01',
            'paused': 0,
            'seeding_time': 13399,
            'last_scrape': 13399,
            'info-hash': '-\xc5\xd0\xe7\x1af\xfeid\x9ad\r9\xcb\x00\xa2YpIs',
            'max_uploads': 16777215,
            'max_connections': 16777215,
            'num_downloaders': 16777215,
            'total_downloaded': 0,
            'file-format': 'libtorrent resume file',
            'peers6': '',
            'added_time': 1411826665,
            'banned_peers6': '',
            'file_priority': [1],
            'last_seen_complete': 0,
            'total_uploaded': 0,
            'piece_priority': '\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01',
            'file-version': 1,
            'announce_to_dht': 1,
            'auto_managed': 1,
            'upload_rate_limit': 0,
            'completed_time': 1411826665,
            'allocation': 'sparse',
            'blocks per piece': 2,
            'download_rate_limit': 0,
            'libtorrent-version': '0.16.17.0',
            'banned_peers': '',
            'num_seeds': 16777215,
            'sequential_download': 0,
            'announce_to_trackers': 1,
            'peers': '\n\x00\x02\x0f=\xc6SC\x17]\xd8}\x7f\x00\x00\x01=\xc6',
            'finished_time': 13399,
            'last_upload': 13399,
            'trackers': [[]],
            'super_seeding': 0,
            'file sizes': [[512000, 1411826586]],
            'last_download': 13399,
        }
        torrent_state = TorrentState(
            torrent_id='2dc5d0e71a66fe69649a640d39cb00a259704973',
            filename='test_torrent.file.torrent',
            name='',
            save_path='/home/ubuntu/Downloads',
            file_priorities=[1],
            is_finished=True,
        )

        filename = common.get_test_data_file('test_torrent.file.torrent')
        with open(filename, 'rb') as _file:
            filedump = _file.read()
        resume_data = utf8_encode_structure(resume_data)
        torrent_id = self.core.torrentmanager.add(
            state=torrent_state, filedump=filedump, resume_data=lt.bencode(resume_data)
        )
        torrent = self.core.torrentmanager.torrents[torrent_id]

        def assert_resume_data():
            self.assert_state(torrent, 'Error')
            tm_resume_data = lt.bdecode(
                self.core.torrentmanager.resume_data[torrent.torrent_id]
            )
            assert tm_resume_data == resume_data

        return deferLater(reactor, 0.5, assert_resume_data)

    def test_get_eta_seeding(self):
        atp = self.get_torrent_atp('test_torrent.file.torrent')
        handle = self.session.add_torrent(atp)
        self.torrent = Torrent(handle, {})
        assert self.torrent.get_eta() == 0
        self.torrent.status = mock.MagicMock()

        self.torrent.status.upload_payload_rate = 5000
        self.torrent.status.download_payload_rate = 0
        self.torrent.status.all_time_download = 10000
        self.torrent.status.all_time_upload = 500
        self.torrent.is_finished = True
        self.torrent.options = {'stop_at_ratio': False}
        # Test finished and uploading but no stop_at_ratio set.
        assert self.torrent.get_eta() == 0

        self.torrent.options = {'stop_at_ratio': True, 'stop_ratio': 1.5}
        result = self.torrent.get_eta()
        assert result == 2
        assert isinstance(result, int)

    def test_get_eta_downloading(self):
        atp = self.get_torrent_atp('test_torrent.file.torrent')
        handle = self.session.add_torrent(atp)
        self.torrent = Torrent(handle, {})
        assert self.torrent.get_eta() == 0

        self.torrent.status = mock.MagicMock()
        self.torrent.status.download_payload_rate = 50
        self.torrent.status.total_wanted = 10000
        self.torrent.status.total_wanted_done = 5000

        result = self.torrent.get_eta()
        assert result == 100
        assert isinstance(result, int)

    def test_get_name_unicode(self):
        """Test retrieving a unicode torrent name from libtorrent."""
        atp = self.get_torrent_atp('unicode_file.torrent')
        handle = self.session.add_torrent(atp)
        self.torrent = Torrent(handle, {})
        assert self.torrent.get_name() == 'সুকুমার রায়.txt'

    def test_rename_unicode(self):
        """Test renaming file/folders with unicode filenames."""
        atp = self.get_torrent_atp('unicode_filenames.torrent')
        handle = self.session.add_torrent(atp)
        self.torrent = Torrent(handle, {})
        # Ignore TorrentManager method call
        TorrentManager.save_resume_data = AsyncMock()

        result = self.torrent.rename_folder('unicode_filenames', 'Горбачёв')
        assert isinstance(result, defer.DeferredList)

        result = self.torrent.rename_files([[0, 'new_рбачёв']])
        assert result is None

    def test_connect_peer_port(self):
        """Test to ensure port is int for libtorrent"""
        atp = self.get_torrent_atp('test_torrent.file.torrent')
        handle = self.session.add_torrent(atp)
        self.torrent = Torrent(handle, {})
        assert not self.torrent.connect_peer('127.0.0.1', 'text')
        assert self.torrent.connect_peer('127.0.0.1', '1234')

    def test_status_cache(self):
        atp = self.get_torrent_atp('test_torrent.file.torrent')
        handle = self.session.add_torrent(atp)
        mock_time = mock.Mock(return_value=time.time())
        with mock.patch('time.time', mock_time):
            torrent = Torrent(handle, {})
            counter = itertools.count()
            handle.status = mock.Mock(side_effect=counter.__next__)
            first_status = torrent.get_lt_status()
            assert first_status == 0, 'sanity check'
            assert first_status == torrent.status, 'cached status should be used'
            assert torrent.get_lt_status() == 1, 'status should update'
            assert torrent.status == 1
            # Advance time and verify cache expires and updates
            mock_time.return_value += 10
            assert torrent.status == 2



#!/usr/bin/env python
#
# Copyright 2014 Calum Lind <calumlind@gmail.com>
#
# This file is part of Deluge and is licensed under GNU General Public License 3.0, or later, with
# the additional special exception to link portions of this program with the OpenSSL library.
# See LICENSE for more details.
#
import os.path
from hashlib import sha256
from subprocess import call, check_output

sdist_formats = 'xztar'

version = check_output(['python', 'version.py']).strip().decode()

# Create release archive
release_dir = 'dist/release-%s' % version
print('Creating release archive for ' + version)

call(
    'python setup.py --quiet egg_info --egg-base /tmp sdist --formats=%s --dist-dir=%s'
    % (sdist_formats, release_dir),
    shell=True,
)


if sdist_formats == 'xztar':
    tarxz_path = os.path.join(release_dir, 'deluge-%s.tar.xz' % version)
else:
    # Compress release archive with xz
    tar_path = os.path.join(release_dir, 'deluge-%s.tar' % version)
    tarxz_path = tar_path + '.xz'
    print('Compressing tar (%s) with xz' % tar_path)
    try:
        from backports import lzma
    except ImportError:
        print('backports.lzma not installed, falling back to xz shell command')
        call(['xz', '-e9zkf', tar_path])
    else:
        with open(tar_path, 'rb') as tar_file, open(tarxz_path, 'wb') as xz_file:
            xz_file.write(
                lzma.compress(bytes(tar_file.read()), preset=9 | lzma.PRESET_EXTREME)
            )

# Calculate shasum and add to sha256sums.txt
with open(tarxz_path, 'rb') as _file:
    sha256sum = '{} {}'.format(
        sha256(_file.read()).hexdigest(),
        os.path.basename(tarxz_path),
    )
with open(os.path.join(release_dir, 'sha256sums.txt'), 'w') as _file:
    _file.write(sha256sum + '\n')

print('Complete: %s' % release_dir)



#!/usr/bin/python
#
# This software is in the public domain, furnished "as is", without technical
# support, and with no warranty, express or implied, as to its usefulness for
# any purpose.
#
# deluge_config.py
# This code (at least in theory) allows one to alter configuration settings
# on a deluge backend.   At the moment, though, it only alters the parameters
# that I've found useful to change.
#
# Authour: Garett Harnish

import logging
import sys
from optparse import OptionParser


def is_float_digit(string):
    if string.isdigit():
        return True
    else:
        try:
            float(string)
            return True
        except ValueError:
            return False


# set up command-line options
parser = OptionParser()
parser.add_option(
    '--port',
    help='port for deluge backend host (default: 58846)',
    default='58846',
    dest='port',
)
parser.add_option(
    '--host',
    help='hostname of deluge backend to connect to (default: localhost)',
    default='localhost',
    dest='host',
)
parser.add_option(
    '--max_active_limit',
    dest='max_active_limit',
    help='sets the absolute maximum number of active torrents on the deluge backend',
)
parser.add_option(
    '--max_active_downloading',
    dest='max_active_downloading',
    help='sets the maximum number of active downloading torrents on the deluge backend',
)
parser.add_option(
    '--max_active_seeding',
    dest='max_active_seeding',
    help='sets the maximum number of active seeding torrents on the deluge backend',
)
parser.add_option(
    '--max_download_speed',
    help='sets the maximum global download speed on the deluge backend',
    dest='max_download_speed',
)
parser.add_option(
    '--max_upload_speed',
    help='sets the maximum global upload speed on the deluge backend',
    dest='max_upload_speed',
)
parser.add_option(
    '--debug',
    help='outputs debug information to the console',
    default=False,
    action='store_true',
    dest='debug',
)

# grab command-line options
(options, args) = parser.parse_args()

if not options.debug:
    logging.disable(logging.ERROR)

settings = {}

# set values if set and valid
if options.max_active_limit:
    if options.max_active_limit.isdigit() and int(options.max_active_limit) >= 0:
        settings['max_active_limit'] = int(options.max_active_limit)
    else:
        sys.stderr.write('ERROR: Invalid max_active_limit parameter!\n')
        sys.exit(-1)

if options.max_active_downloading:
    if (
        options.max_active_downloading.isdigit()
        and int(options.max_active_downloading) >= 0
    ):
        settings['max_active_downloading'] = int(options.max_active_downloading)
    else:
        sys.stderr.write('ERROR: Invalid max_active_downloading parameter!\n')
        sys.exit(-1)

if options.max_active_seeding:
    if options.max_active_seeding.isdigit() and int(options.max_active_seeding) >= 0:
        settings['max_active_seeding'] = int(options.max_active_seeding)
    else:
        sys.stderr.write('ERROR: Invalid max_active_seeding parameter!\n')
        sys.exit(-1)

if options.max_download_speed:
    if is_float_digit(options.max_download_speed) and (
        float(options.max_download_speed) >= 0.0
        or float(options.max_download_speed) == -1.0
    ):
        settings['max_download_speed'] = float(options.max_download_speed)
    else:
        sys.stderr.write('ERROR: Invalid max_download_speed parameter!\n')
        sys.exit(-1)

if options.max_upload_speed:
    if is_float_digit(options.max_upload_speed) and (
        float(options.max_upload_speed) >= 0.0
        or float(options.max_upload_speed) == -1.0
    ):
        settings['max_upload_speed'] = float(options.max_upload_speed)
    else:
        sys.stderr.write('ERROR: Invalid max_upload_speed parameter!\n')
        sys.exit(-1)

# If there is something to do ...
if settings:
    # create connection to daemon
    from deluge.ui.client import sclient as client

    client.set_core_uri('http://' + options.host + ':' + options.port)

    # commit configurations changes
    client.set_config(settings)



#
# Deluge documentation build configuration file
#
# This file is execfile()d with the current directory set to its containing dir.
#
# The contents of this file are pickled, so don't put values in the namespace
# that aren't pickleable (module imports are okay, they're removed automatically).
#
# All configuration values have a default value; values that are commented out
# serve to show the default value.

import builtins
import os
import sys
from datetime import date

from sphinx.ext import apidoc
from sphinx.ext.autodoc import ClassDocumenter, bool_option

# If your extensions are in another directory, add it here. If the directory is relative
# to the documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.append(
    os.path.abspath(
        os.path.join(
            os.path.join(os.path.dirname(__file__), os.path.pardir), os.path.pardir
        )
    )
)
# Importing version only possible after add project root to sys.path.
try:
    from version import get_version
except ImportError:
    from deluge.common import get_version


# General configuration
# ---------------------

needs_sphinx = '2.0'
suppress_warnings = ['app.add_source_parser']

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.napoleon',
    'sphinx.ext.coverage',
    'sphinxcontrib.spelling',
    'myst_parser',
    'sphinx_autodoc_typehints',
]

napoleon_include_init_with_doc = True
napoleon_use_rtype = False

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = ['.rst', '.md']

# The master toctree document.
master_doc = 'index'

# General substitutions.
project = 'Deluge'
current_year = date.today().year
copyright = '2008-%s, Deluge Team' % current_year  # noqa: A001

# The full version, including alpha/beta/rc tags.
release = get_version()
# The short X.Y version.
version = '.'.join(release.split('.', 2)[:2])

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
today_fmt = '%B %d, %Y'

# List of documents that shouldn't be included in the build.
# unused_docs = []

# List of directories, relative to source directories, that shouldn't be searched
# for source files.
exclude_patterns = []

# The reST default role (used for this markup: `text`) to use for all documents.
# default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
# add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'


# Options for spelling
# --------------------
spelling_show_suggestions = True
spelling_word_list_filename = '../spelling_wordlist.txt'
# Skip Deluge module rst files
if 'spelling' in sys.argv or 'spellcheck_docs' in sys.argv:
    exclude_patterns += ['modules']

# Options for HTML output
# -----------------------
html_theme = 'sphinx_rtd_theme'
# The style sheet to use for HTML and HTML Help pages. A file of that name
# must exist either in Sphinx' static/ path, or in one of the custom paths
# given in html_static_path.
# html_style = 'default.css'

# Add font-mfizz for icons.
html_css_files = [
    'https://cdnjs.cloudflare.com/ajax/libs/font-mfizz/2.4.1/font-mfizz.min.css'
]
# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
# html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
# html_short_title = None

# The name of an image file (within the static path) to place at the top of
# the sidebar.
# html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = '../../deluge/ui/data/pixmaps/deluge.ico'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ['_static']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
# html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
# html_additional_pages = {}

# If false, no module index is generated.
# html_use_modindex = True

# If false, no index is generated.
# html_use_index = True

# If true, the index is split into individual pages for each letter.
# html_split_index = False

# If true, the reST sources are included in the HTML build as _sources/<name>.
# html_copy_source = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
# html_use_opensearch = ''

# If nonempty, this is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = ''

# Output file base name for HTML help builder.
htmlhelp_basename = 'delugedoc'


# Options for LaTeX output
# ------------------------

# The paper size ('letter' or 'a4').
# latex_paper_size = 'letter'

# The font size ('10pt', '11pt' or '12pt').
# latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, document class [howto/manual]).
latex_documents = [
    ('index', 'deluge.tex', 'deluge Documentation', 'Deluge Team', 'manual')
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# Additional stuff for the LaTeX preamble.
# latex_preamble = ''

# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
# latex_use_modindex = True


# Autodoc section
# ---------------

# Must add these for autodoc to import packages successfully
builtins.__dict__['_'] = lambda x: x
builtins.__dict__['_n'] = lambda s, p, n: s if n == 1 else p

autodoc_mock_imports = [
    'deluge._libtorrent',
    'twisted',
    'rencode',
    'OpenSSL',
    'PIL',
    'libtorrent',
    'psyco',
    'gi',
    'cairo',
    'curses',
    'win32api',
    'win32file',
    'win32process',
    'win32pipe',
    'pywintypes',
    'win32con',
    'win32event',
    'pytest',
    'mock',
    'mako',
    'xdg',
    'zope',
    'zope.interface',
]

# Register an autodoc class directive to only include exported methods.
ClassDocumenter.option_spec['exported'] = bool_option


def maybe_skip_member(app, what, name, obj, skip, options):
    if options.exported and not (
        hasattr(obj, '_rpcserver_export') or hasattr(obj, '_json_export')
    ):
        return True


# Run the sphinx-apidoc to create package/modules rst files for autodoc.
def run_apidoc(__):
    cur_dir = os.path.abspath(os.path.dirname(__file__))
    module_dir = os.path.join(cur_dir, '..', '..', 'deluge')
    ignore_paths = [
        os.path.join(module_dir, 'plugins'),
        os.path.join(module_dir, 'tests'),
    ]
    argv = [
        '--force',
        '--no-toc',
        '--output-dir',
        os.path.join(cur_dir, 'modules'),
        module_dir,
    ] + ignore_paths
    apidoc.main(argv)


def setup(app):
    app.connect('builder-inited', run_apidoc)
    app.connect('autodoc-skip-member', maybe_skip_member)



#
# Copyright (C) 2008 Martijn Voncken <mvoncken@gmail.com>
#
# This file is part of Deluge and is licensed under GNU General Public License 3.0, or later, with
# the additional special exception to link portions of this program with the OpenSSL library.
# See LICENSE for more details.
#

from setuptools import find_packages, setup

__plugin_name__ = 'Label'
__author__ = 'Martijn Voncken'
__author_email__ = 'mvoncken@gmail.com'
__version__ = '0.3'
__url__ = 'http://deluge-torrent.org'
__license__ = 'GPLv3'
__description__ = 'Allows labels to be assigned to torrents'
__long_description__ = """
Allows labels to be assigned to torrents

Also offers filters on state, tracker and keywords
"""
__pkg_data__ = {'deluge_' + __plugin_name__.lower(): ['data/*']}

setup(
    name=__plugin_name__,
    version=__version__,
    description=__description__,
    author=__author__,
    author_email=__author_email__,
    url=__url__,
    license=__license__,
    long_description=__long_description__,
    packages=find_packages(),
    package_data=__pkg_data__,
    entry_points="""
    [deluge.plugin.core]
    %s = deluge_%s:CorePlugin
    [deluge.plugin.gtk3ui]
    %s = deluge_%s:GtkUIPlugin
    [deluge.plugin.web]
    %s = deluge_%s:WebUIPlugin
    """
    % ((__plugin_name__, __plugin_name__.lower()) * 3),
)



#!/usr/bin/env python
#
# Copyright (C) 2012-2015 Calum Lind <calumlind@gmail.com>
# Copyright (C) 2010 Damien Churchill <damoxc@gmail.com>
# Copyright (C) 2009-2010 Andrew Resch <andrewresch@gmail.com>
# Copyright (C) 2009 Jesper Lund <mail@jesperlund.com>
#
# This file is part of Deluge and is licensed under GNU General Public License 3.0, or later, with
# the additional special exception to link portions of this program with the OpenSSL library.
# See LICENSE for more details.
#

import os

import deluge.common

# Get build_version from installed deluge.
build_version = deluge.common.get_version()
build_dir = os.path.join('freeze', 'Deluge')

# Copy version info to file for nsis script.
with open('VERSION.tmp', 'w') as ver_file:
    ver_file.write('build_version = "%s"' % build_version)

# Create the install and uninstall file list for NSIS.
filedir_list = []
for root, dirnames, filenames in os.walk(build_dir):
    dirnames.sort()
    filenames.sort()
    filedir_list.append((root[len(build_dir) :], filenames))

with open('install_files.nsh', 'w') as f:
    f.write('; Files to install\n')
    for dirname, files in filedir_list:
        if not dirname:
            dirname = os.sep
        f.write('\nSetOutPath "$INSTDIR%s"\n' % dirname)
        for filename in files:
            f.write('File ' + build_dir + os.path.join(dirname, filename) + '\n')

with open('uninstall_files.nsh', 'w') as f:
    f.write('; Files to uninstall\n')
    for dirname, files in reversed(filedir_list):
        f.write('\n')
        if not dirname:
            dirname = os.sep
        for filename in files:
            f.write('Delete "$INSTDIR%s"\n' % os.path.join(dirname, filename))
        f.write('RMDir "$INSTDIR%s"\n' % dirname)



#
# Copyright (C) 2016 bendikro <bro.devel+deluge@gmail.com>
#
# This file is part of Deluge and is licensed under GNU General Public License 3.0, or later, with
# the additional special exception to link portions of this program with the OpenSSL library.
# See LICENSE for more details.
#

import json as json_lib
from io import BytesIO

import pytest
import twisted.web.client
from twisted.internet import reactor
from twisted.web.client import Agent, FileBodyProducer
from twisted.web.http_headers import Headers

from . import common
from .common import get_test_data_file
from .common_web import WebServerMockBase, WebServerTestBase

common.disable_new_release_check()


class TestWebServer(WebServerTestBase, WebServerMockBase):
    async def test_get_torrent_info(self):
        agent = Agent(reactor)

        self.mock_authentication_ignore(self.deluge_web.auth)

        # This torrent file contains an uncommon field 'filehash' which must be hex
        # encoded to allow dumping the torrent info to json. Otherwise it will fail with:
        # UnicodeDecodeError: 'utf8' codec can't decode byte 0xe5 in position 0: invalid continuation byte
        filename = get_test_data_file('filehash_field.torrent')
        input_file = (
            '{"params": ["%s"], "method": "web.get_torrent_info", "id": 22}'
            % filename.replace('\\', '\\\\')
        )
        headers = {
            b'User-Agent': ['Twisted Web Client Example'],
            b'Content-Type': ['application/json'],
        }
        url = 'http://127.0.0.1:%s/json' % self.deluge_web.port

        response = await agent.request(
            b'POST',
            url.encode(),
            Headers(headers),
            FileBodyProducer(BytesIO(input_file.encode())),
        )
        body = await twisted.web.client.readBody(response)

        try:
            json = json_lib.loads(body.decode())
        except Exception:
            print('aoeu')
        assert json['error'] is None
        assert 'torrent_filehash' == json['result']['name']

    @pytest.mark.parametrize('base', ['', '/', 'deluge'])
    async def test_base_with_config(self, base):
        agent = Agent(reactor)
        root_url = f'http://127.0.0.1:{self.deluge_web.port}'
        base_url = f'{root_url}/{base}'

        self.deluge_web.base = base

        response = await agent.request(b'GET', root_url.encode())
        assert response.code == 200
        body = await twisted.web.client.readBody(response)
        assert 'Deluge WebUI' in body.decode()

        response = await agent.request(b'GET', base_url.encode())
        assert response.code == 200

    @pytest.mark.parametrize('base', ['/', 'deluge'])
    async def test_base_with_config_recurring_basepath(self, base):
        agent = Agent(reactor)
        base_url = f'http://127.0.0.1:{self.deluge_web.port}/{base}'

        self.deluge_web.base = base

        response = await agent.request(b'GET', base_url.encode())
        assert response.code == 200

        recursive_url = f'{base_url}/{base}'
        response = await agent.request(b'GET', recursive_url.encode())
        assert response.code == 404 if base.strip('/') else 200

        recursive_url = f'{recursive_url}/{base}'
        response = await agent.request(b'GET', recursive_url.encode())
        assert response.code == 404 if base.strip('/') else 200

    async def test_base_with_deluge_header(self):
        """Ensure base path is set and HTML contains path"""
        agent = Agent(reactor)
        base = 'deluge'
        url = f'http://127.0.0.1:{self.deluge_web.port}'
        headers = Headers({'X-Deluge-Base': [base]})

        response = await agent.request(b'GET', url.encode(), headers)
        body = await twisted.web.client.readBody(response)
        assert f'href="/{base}/' in body.decode()

        # Header only changes HTML base path so ensure no resource at server path
        url = f'{url}/{base}'
        response = await agent.request(b'GET', url.encode(), headers)
        assert response.code == 404



#
# This file is part of Deluge and is licensed under GNU General Public License 3.0, or later, with
# the additional special exception to link portions of this program with the OpenSSL library.
# See LICENSE for more details.
#
import sys
from dataclasses import dataclass

import pytest

from deluge.core.core import Core


class LtSessionMock:
    def __init__(self):
        self.alerts = []

    def push_alerts(self, alerts):
        self.alerts = alerts

    def wait_for_alert(self, timeout):
        return self.alerts[0] if len(self.alerts) > 0 else None

    def pop_alerts(self):
        alerts = self.alerts
        self.alerts = []
        return alerts


@dataclass
class LtAlertMock:
    type: int
    name: str
    message: str

    def message(self):
        return self.message

    def what(self):
        return self.name


@pytest.fixture
def mock_alert1():
    return LtAlertMock(type=1, name='mock_alert1', message='Alert 1')


@pytest.fixture
def mock_alert2():
    return LtAlertMock(type=2, name='mock_alert2', message='Alert 2')


class TestAlertManager:
    @pytest.fixture(autouse=True)
    def set_up(self, component):
        self.core = Core()
        self.core.config.config['lsd'] = False
        self.am = component.get('AlertManager')
        self.am.session = LtSessionMock()

        component.start(['AlertManager'])

    def test_register_handler(self):
        def handler(alert): ...

        self.am.register_handler('dummy1', handler)
        self.am.register_handler('dummy2_alert', handler)
        assert self.am.handlers['dummy1'] == [handler]
        assert self.am.handlers['dummy2'] == [handler]

    async def test_pop_alert(self, mock_callback, mock_alert1, mock_alert2):
        self.am.register_handler('mock_alert1', mock_callback)

        self.am.session.push_alerts([mock_alert1, mock_alert2])

        await mock_callback.deferred

        mock_callback.assert_called_once_with(mock_alert1)

    @pytest.mark.xfail(
        sys.platform == 'win32',
        reason='Issue under Windows where mock is already called.',
    )
    async def test_pause_not_pop_alert(
        self, component, mock_alert1, mock_alert2, mock_callback
    ):
        await component.pause(['AlertManager'])

        self.am.register_handler('mock_alert1', mock_callback)
        self.am.session.push_alerts([mock_alert1, mock_alert2])

        await mock_callback.deferred

        mock_callback.assert_not_called()
        assert not self.am._event.is_set()
        assert len(self.am.session.alerts) == 2

    def test_deregister_handler(self):
        def handler(alert): ...

        self.am.register_handler('dummy1', handler)
        self.am.register_handler('dummy2_alert', handler)
        self.am.deregister_handler(handler)
        assert self.am.handlers['dummy1'] == []
        assert self.am.handlers['dummy2'] == []



"""
Creates an empty plugin and links it from ~/.config/deluge/plugins
This plugin includes the framework for using the preferences dialog

example:
python create_plugin.py --name MyPlugin2 --basepath . --author-name "Your Name" --author-email "yourname@example.com"

"""

import os
import sys
from argparse import ArgumentParser
from datetime import datetime

import deluge.common

parser = ArgumentParser()
parser.add_argument(
    '-n', '--name', metavar='<plugin name>', required=True, help='Plugin name'
)
parser.add_argument('-m', '--module-name', metavar='<module name>', help='Module name')
parser.add_argument(
    '-p', '--basepath', metavar='<path>', required=True, help='Base path'
)
parser.add_argument(
    '-a',
    '--author-name',
    metavar='<author name>',
    required=True,
    help='Author name,for the GPL header',
)
parser.add_argument(
    '-e',
    '--author-email',
    metavar='<author email>',
    required=True,
    help='Author email,for the GPL header',
)
parser.add_argument('-u', '--url', metavar='<URL>', help='Homepage URL')
parser.add_argument(
    '-c',
    '--config',
    metavar='<Config dir>',
    dest='configdir',
    help='Location of deluge configuration',
)

options = parser.parse_args()


def create_plugin():
    if not options.url:
        options.url = ''

    if not os.path.exists(options.basepath):
        print('basepath does not exist')
        return

    if not options.configdir:
        options.configdir = deluge.common.get_default_config_dir()

    options.configdir = os.path.realpath(options.configdir)

    real_name = options.name
    name = real_name.replace(' ', '_')
    safe_name = name.lower()
    if options.module_name:
        safe_name = options.module_name.lower()
    plugin_base = os.path.realpath(os.path.join(options.basepath, name))
    src = os.path.join(plugin_base, 'deluge_' + safe_name)
    data_dir = os.path.join(src, 'data')
    python_path = sys.executable

    if os.path.exists(plugin_base):
        print('the directory %s already exists, delete it first' % plugin_base)
        return

    def write_file(path, filename, template, include_gpl=True):
        plugin_args = {
            'author_name': options.author_name,
            'author_email': options.author_email,
            'name': name,
            'safe_name': safe_name,
            'filename': filename,
            'plugin_base': plugin_base,
            'python_path': python_path,
            'url': options.url,
            'configdir': options.configdir,
            'current_year': datetime.utcnow().year,
        }

        filename = os.path.join(path, filename)
        with open(filename, 'w') as _file:
            if filename.endswith('.py') and include_gpl:
                _file.write(GPL % plugin_args)
            _file.write(template % plugin_args)

    print('creating folders..')
    os.mkdir(plugin_base)
    os.mkdir(src)
    os.mkdir(data_dir)

    print('creating files..')
    write_file(plugin_base, 'setup.py', SETUP)
    write_file(src, '__init__.py', INIT)
    write_file(src, 'gtk3ui.py', GTK3UI)
    write_file(src, 'webui.py', WEBUI)
    write_file(src, 'core.py', CORE)
    write_file(src, 'common.py', COMMON)
    write_file(data_dir, 'config.ui', GLADE)
    write_file(data_dir, '%s.js' % safe_name, DEFAULT_JS)

    # add an input parameter for this?
    print('building dev-link..')
    if deluge.common.windows_check():
        write_file(plugin_base, 'create_dev_link.bat', CREATE_DEV_LINK_WIN)
        dev_link_path = os.path.join(plugin_base, 'create_dev_link.bat')
    else:
        write_file(plugin_base, 'create_dev_link.sh', CREATE_DEV_LINK_NIX)
        dev_link_path = os.path.join(plugin_base, 'create_dev_link.sh')
        os.system('chmod +x %s' % dev_link_path)  # lazy..
    os.system(dev_link_path)


CORE = """from __future__ import unicode_literals

import logging

import deluge.configmanager
from deluge.core.rpcserver import export
from deluge.plugins.pluginbase import CorePluginBase

log = logging.getLogger(__name__)

DEFAULT_PREFS = {
    'test': 'NiNiNi'
}


class Core(CorePluginBase):
    def enable(self):
        self.config = deluge.configmanager.ConfigManager(
            '%(safe_name)s.conf', DEFAULT_PREFS)

    def disable(self):
        pass

    def update(self):
        pass

    @export
    def set_config(self, config):
        \"\"\"Sets the config dictionary\"\"\"
        for key in config:
            self.config[key] = config[key]
        self.config.save()

    @export
    def get_config(self):
        \"\"\"Returns the config dictionary\"\"\"
        return self.config.config
"""

INIT = """from deluge.plugins.init import PluginInitBase


class CorePlugin(PluginInitBase):
    def __init__(self, plugin_name):
        from .core import Core as PluginClass
        self._plugin_cls = PluginClass
        super(CorePlugin, self).__init__(plugin_name)


class Gtk3UIPlugin(PluginInitBase):
    def __init__(self, plugin_name):
        from .gtk3ui import Gtk3UI as PluginClass
        self._plugin_cls = PluginClass
        super(Gtk3UIPlugin, self).__init__(plugin_name)


class WebUIPlugin(PluginInitBase):
    def __init__(self, plugin_name):
        from .webui import WebUI as PluginClass
        self._plugin_cls = PluginClass
        super(WebUIPlugin, self).__init__(plugin_name)
"""


SETUP = """from setuptools import find_packages, setup

__plugin_name__ = '%(name)s'
__author__ = '%(author_name)s'
__author_email__ = '%(author_email)s'
__version__ = '0.1'
__url__ = '%(url)s'
__license__ = 'GPLv3'
__description__ = ''
__long_description__ = \"\"\"\"\"\"
__pkg_data__ = {'deluge_'+__plugin_name__.lower(): ['data/*']}

setup(
    name=__plugin_name__,
    version=__version__,
    description=__description__,
    author=__author__,
    author_email=__author_email__,
    url=__url__,
    license=__license__,
    long_description=__long_description__,

    packages=find_packages(),
    package_data=__pkg_data__,

    entry_points=\"\"\"
    [deluge.plugin.core]
    %%s = deluge_%%s:CorePlugin
    [deluge.plugin.gtk3ui]
    %%s = deluge_%%s:Gtk3UIPlugin
    [deluge.plugin.web]
    %%s = deluge_%%s:WebUIPlugin
    \"\"\" %% ((__plugin_name__, __plugin_name__.lower()) * 3)
)
"""

COMMON = """from __future__ import unicode_literals

import os.path

from pkg_resources import resource_filename


def get_resource(filename):
    return resource_filename(__package__, os.path.join('data', filename))
"""

GTK3UI = """from __future__ import unicode_literals

import logging

from gi.repository import Gtk

import deluge.component as component
from deluge.plugins.pluginbase import Gtk3PluginBase
from deluge.ui.client import client

from .common import get_resource

log = logging.getLogger(__name__)


class Gtk3UI(Gtk3PluginBase):
    def enable(self):
        self.builder = Gtk.Builder()
        self.builder.add_from_file(get_resource('config.ui'))

        component.get('Preferences').add_page(
            '%(name)s', self.builder.get_object('prefs_box'))
        component.get('PluginManager').register_hook(
            'on_apply_prefs', self.on_apply_prefs)
        component.get('PluginManager').register_hook(
            'on_show_prefs', self.on_show_prefs)

    def disable(self):
        component.get('Preferences').remove_page('%(name)s')
        component.get('PluginManager').deregister_hook(
            'on_apply_prefs', self.on_apply_prefs)
        component.get('PluginManager').deregister_hook(
            'on_show_prefs', self.on_show_prefs)

    def on_apply_prefs(self):
        log.debug('applying prefs for %(name)s')
        config = {
            'test': self.builder.get_object('txt_test').get_text()
        }
        client.%(safe_name)s.set_config(config)

    def on_show_prefs(self):
        client.%(safe_name)s.get_config().addCallback(self.cb_get_config)

    def cb_get_config(self, config):
        \"\"\"callback for on show_prefs\"\"\"
        self.builder.get_object('txt_test').set_text(config['test'])
"""

GLADE = """<?xml version="1.0" encoding="UTF-8"?>
<!-- Generated with glade 3.18.3 -->
<interface>
  <requires lib="gtk+" version="3.0"/>
  <object class="GtkWindow" id="window1">
    <child>
      <object class="GtkBox" id="prefs_box">
        <property name="visible">True</property>
        <child>
          <object class="GtkLabel" id="label1">
            <property name="visible">True</property>
            <property name="label" translatable="yes">Test config value:</property>
          </object>
        </child>
        <child>
          <object class="GtkEntry" id="txt_test">
            <property name="visible">True</property>
            <property name="can_focus">True</property>
          </object>
          <packing>
            <property name="position">1</property>
          </packing>
        </child>
      </object>
    </child>
  </object>
</interface>
"""

WEBUI = """from __future__ import unicode_literals

import logging

from deluge.plugins.pluginbase import WebPluginBase

from .common import get_resource

log = logging.getLogger(__name__)


class WebUI(WebPluginBase):

    scripts = [get_resource('%(safe_name)s.js')]

    def enable(self):
        pass

    def disable(self):
        pass
"""

DEFAULT_JS = """/**
 * Script: %(filename)s
 *     The client-side javascript code for the %(name)s plugin.
 *
 * Copyright:
 *     (C) %(author_name)s %(current_year)s <%(author_email)s>
 *
 *     This file is part of %(name)s and is licensed under GNU GPL 3.0, or
 *     later, with the additional special exception to link portions of this
 *     program with the OpenSSL library. See LICENSE for more details.
 */

%(name)sPlugin = Ext.extend(Deluge.Plugin, {
    constructor: function(config) {
        config = Ext.apply({
            name: '%(name)s'
        }, config);
        %(name)sPlugin.superclass.constructor.call(this, config);
    },

    onDisable: function() {
        deluge.preferences.removePage(this.prefsPage);
    },

    onEnable: function() {
        this.prefsPage = deluge.preferences.addPage(
            new Deluge.ux.preferences.%(name)sPage());
    }
});
new %(name)sPlugin();
"""

GPL = """# -*- coding: utf-8 -*-
# Copyright (C) %(current_year)d %(author_name)s <%(author_email)s>
#
# Basic plugin template created by the Deluge Team.
#
# This file is part of %(name)s and is licensed under GNU GPL 3.0, or later,
# with the additional special exception to link portions of this program with
# the OpenSSL library. See LICENSE for more details.
"""

CREATE_DEV_LINK_NIX = """#!/bin/bash
BASEDIR=$(cd `dirname $0` && pwd)
CONFIG_DIR=$( test -z $1 && echo "%(configdir)s" || echo "$1")
[ -d "$CONFIG_DIR/plugins" ] || echo "Config dir \"$CONFIG_DIR\" is either not a directory \
or is not a proper deluge config directory. Exiting"
[ -d "$CONFIG_DIR/plugins" ] || exit 1
cd $BASEDIR
test -d $BASEDIR/temp || mkdir $BASEDIR/temp
export PYTHONPATH=$BASEDIR/temp
%(python_path)s setup.py build develop --install-dir $BASEDIR/temp
cp $BASEDIR/temp/*.egg-link $CONFIG_DIR/plugins
rm -fr $BASEDIR/temp
"""

CREATE_DEV_LINK_WIN = """@echo off
set BASEDIR=%%~dp0
set BASEDIR=%%BASEDIR:~0,-1%%
if [%%1]==[] (
    set CONFIG_DIR=%(configdir)s
) else (
    set CONFIG_DIR=%%1
)
if not exist %%CONFIG_DIR%%\\plugins (
    echo Config dir %%CONFIG_DIR%% is either not a directory \
or is not a proper deluge config directory. Exiting
    exit /b 1
)
cd %%BASEDIR%%
if not exist %%BASEDIR%%\\temp (
    md %%BASEDIR%%\\temp
)
set PYTHONPATH=%%BASEDIR%%/temp
%(python_path)s setup.py build develop --install-dir %%BASEDIR%%\\temp
copy "%%BASEDIR%%\\temp\\*.egg-link" "%%CONFIG_DIR%%\\plugins"
rd /s /q %%BASEDIR%%\\temp
"""

create_plugin()



#
# Copyright (C) 2009 Damien Churchill <damoxc@gmail.com>
#
# This file is part of Deluge and is licensed under GNU General Public License 3.0, or later, with
# the additional special exception to link portions of this program with the OpenSSL library.
# See LICENSE for more details.
#

import logging

from deluge.plugins.pluginbase import WebPluginBase

from .common import get_resource

log = logging.getLogger(__name__)


class WebUI(WebPluginBase):
    scripts = [get_resource('execute.js')]
    debug_scripts = scripts
